<?php
// Part of the Tab Override WordPress plugin

/**
 * Contains the Tab_Override class
 *
 * @package Tab_Override
 */

/**
 * The Tab_Override class
 *
 * This class contains the main functionality of the Tab Override plugin.
 * This class follows the singleton pattern: it can only be instantiated once.
 * To get an instance of the class, call Tab_Override::get_instance( $plugin_file_path ).
 * This class requires PHP 5.
 *
 * @package Tab_Override
 */
class Tab_Override {
	
	protected static $_instance; // an instance of the class
	protected $_plugin_file; // the full path of the main plugin file
	protected $_initialized = false; // whether the _init_hook() method was called
	protected $_options; // the array of options from the database
	
	/**
	 * Initializes variables, loads the textdomain and adds initial actions
	 * and filters for the plugin.
	 *
	 * @param string $plugin_file_path the full path of the main plugin file
	 */
	protected function __construct( $plugin_file_path ) {
		$this->_plugin_file = $plugin_file_path;
		
		register_activation_hook( $this->_plugin_file, array( $this, 'set_defaults' ) );
		load_plugin_textdomain( 'tab-override', false, basename( dirname( $this->_plugin_file ) ) . '/languages/' );
		
		// create the tab_override_init hook after all plugins are loaded
		add_action( 'plugins_loaded', array( $this, '_init_hook' ) );
		
		add_action( 'admin_menu', array( $this, 'create_menu' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'add_scripts' ) );
		
		add_filter( 'plugin_action_links', array( $this, 'set_action_links' ), 10, 2 );
		add_filter( 'plugin_row_meta', array( $this, 'set_meta_links' ), 10, 2 );
	}
	
	/**
	 * Returns an instance of the Tab_Override class.
	 *
	 * @param  string         $plugin_file_path the full path of the main plugin file
	 * @return object|boolean                   an instance of the Tab_Override class
	 *                                          or false if one cannot be created
	 */
	public static function get_instance( $plugin_file_path ) {
		// only create a new instance if one does not already exist and the
		// plugin file path is known
		if ( ! self::$_instance ) {
			if ( ! $plugin_file_path ) {
				return false;
			}
			self::$_instance = new self( $plugin_file_path );
		}
		
		return self::$_instance;
	}
	
	/**
	 * Executes any actions added to the tab_override_init hook. This method
	 * is added to the plugins_loaded hook so other plugins have a chance to
	 * add actions to the tab_override_init hook. This method must be public
	 * in order to be used as a callback for add_action().
	 */
	public function _init_hook() {
		// only call _init_hook() once
		if ( $this->_initialized ) {
			return;
		}
		$this->_initialized = true;
		
		// create a hook to allow for additional init actions
		do_action( 'tab_override_init' );
	}
	
	/**
	 * Checks the WordPress version and sets default values for the options.
	 */
	public function set_defaults() {
		// don't activate plugin if WordPress version is less than 2.8
		if ( version_compare( get_bloginfo( 'version' ), '2.8', '<' ) ) {
			
			deactivate_plugins( plugin_basename( $this->_plugin_file ) );
			wp_die( esc_html__(
				'The Tab Override plugin requires WordPress version 2.8 or higher.',
				'tab-override'
			) );
		}
		
		// delete and re-create the option in order to turn off autoload (must use add_option)
		// in previous versions, autoload was on
		$settings = get_option( 'tab_override' );
		delete_option( 'tab_override' );
		$default_tab_size = '';
		
		// first time plugin is activated
		if ( ! is_array( $settings ) ) {
			// get the old setting if present and remove it
			$old_tab_size = get_option( 'tab_override_tab_size' );
			delete_option( 'tab_override_tab_size' );
			
			// use the old tab size setting if possible
			// no need to check for the empty string since it's the default anyway
			if ( is_numeric( $old_tab_size ) ) {
				$old_tab_size = absint( $old_tab_size );
				if ( $old_tab_size ) {
					$default_tab_size = ( 100 < $old_tab_size ) ? 100 : $old_tab_size;
				}
			}
		}
		
		// set the defaults when $settings is empty (first time plugin is activated)
		// and whenever the plugin is activated with the reset_options setting checked
		if ( ! is_array( $settings ) || $settings['reset_options'] ) {
			$settings = array(
				'tab_size' => $default_tab_size,
				'auto_indent' => 1,
				'enable_index' => 1,
				'enable_post' => 1,
				'enable_post_new' => 1,
				'enable_page' => 1,
				'enable_page_new' => 1,
				'enable_edit_comments' => 1,
				'enable_comment' => 1,
				'enable_theme_editor' => 1,
				'enable_plugin_editor' => 1,
				'reset_options' => 0
			);
		}
		
		add_option( 'tab_override', $settings, '', 'no' );
	}
	
	/**
	 * Getter method for the main plugin file path.
	 *
	 * @return string the full path of the main plugin file
	 */
	public function get_plugin_file() {
		return $this->_plugin_file;
	}
	
	/**
	 * Adds a settings link to the plugins page.
	 *
	 * @param  array  $links the action links for the plugin (Activate, Deactivate, Edit, Delete)
	 * @param  string $file  the plugin file name
	 * @return array         the modified links array
	 */
	public function set_action_links( $links, $file ) {
		if ( $file == plugin_basename( $this->_plugin_file ) ) {
			array_unshift( $links, sprintf(
				'<a title="%1$s" href="options-general.php?page=tab-override-settings">%2$s</a>',
				esc_attr__( 'Edit settings for this plugin', 'tab-override' ),
				esc_html__( 'Settings', 'tab-override' )
			) );
		}
		
		return $links;
	}
	
	/**
	 * Changes the author site link on the plugins page to one link per author.
	 *
	 * @param  array  $links the meta links for the plugin (author site, plugin site)
	 * @param  string $file  the plugin file name
	 * @return array         the modified links array
	 */
	public function set_meta_links( $links, $file ) {
		if ( $file == plugin_basename( $this->_plugin_file ) ) {
			// look for author link - split text into two links
			foreach ( $links as $key => $link ) {
				if ( strpos( $link, 'author' ) !== false ) {
					$links[$key] = sprintf(
						'By <a title="%1$s" href="http://wjbryant.com/">Bill Bryant</a>, <a title="%1$s" href="http://tinsology.net/">Mathew Tinsley</a>',
						esc_attr__( 'Visit author homepage' )
					);
					break;
				}
			}
		}
		
		return $links;
	}
	
	/**
	 * Outputs the HTML for the plugin settings page.
	 */
	public function create_settings_page() {
		// get the current options for the field methods
		$this->_options = get_option( 'tab_override' );
		?>
		<div class="wrap">
		<?php screen_icon(); ?>
		<h2><?php esc_html_e( 'Tab Override Settings', 'tab-override' ); ?></h2>
		<form method="post" action="options.php">
			<?php
				settings_fields( 'tab_override' );
				do_settings_sections( 'tab-override-settings' );
			?>
			<p class="submit">
				<input type="submit" class="button-primary" value="<?php esc_attr_e( 'Save Changes', 'tab-override' ); ?>" />
			</p>
			<div style="text-align: right;">
				<a title="<?php esc_attr_e( 'Tab Override Documentation at Tinsology.net', 'tab-override' ); ?>" href="http://tinsology.net/plugins/tab-override/"><?php esc_html_e( 'Documentation', 'tab-override' ); ?></a> |
				<a title="<?php esc_attr_e( 'Tab Override Page at WJBryant.com', 'tab-override' ); ?>" href="http://wjbryant.com/projects/tab-override/">WJBryant.com</a>
			</div>
		</form>
		</div>
		<?php
	}
	
	/**
	 * Outputs the description HTML for the editor settings section.
	 */
	public function editor_section_description() {
		echo '<p>These options control how the tab and enter keys function in the HTML editor.</p>';
	}
	
	/**
	 * Outputs the HTML for the tab size field.
	 */
	public function tab_size_field() {
		?>
		<input type="text" id="tab_override_tab_size" maxlength="3" class="small-text" name="tab_override[tab_size]" value="<?php echo esc_attr( $this->_options['tab_size'] ); ?>" />
		<span class="description"><?php esc_html_e( 'Leave blank to use the tab character (default)', 'tab-override' ); ?></span>
		<?php
	}
	
	/**
	 * Outputs the HTML for the auto indent field.
	 */
	public function auto_indent_field() {
		?>
		<input type="checkbox" id="tab_override_auto_indent" name="tab_override[auto_indent]" value="1" <?php checked( $this->_options['auto_indent'], 1 ); ?> />
		<label for="tab_override_auto_indent"><?php esc_html_e( 'Enable auto indent', 'tab-override' ); ?></label>
		<?php
	}
	
	/**
	 * Outputs the description HTML for the general settings section.
	 */
	public function general_section_description() {
		// no description
	}
	
	/**
	 * Outputs the HTML for the enable fields.
	 */
	public function enable_fields() {
		$is_version_3_plus = version_compare( get_bloginfo( 'version' ), '3', '>=' );
		?>
		<input type="checkbox" id="tab_override_enable_index" name="tab_override[enable_index]" value="1" <?php checked( $this->_options['enable_index'], 1 ); ?> />
		<label for="tab_override_enable_index"><?php esc_html_e( 'Dashboard (QuickPress)', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_post" name="tab_override[enable_post]" value="1" <?php checked( $this->_options['enable_post'], 1 ); ?> />
		<label for="tab_override_enable_post"><?php $is_version_3_plus ? esc_html_e( 'Post / Page', 'tab-override' ) : esc_html_e( 'Post', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_post_new" name="tab_override[enable_post_new]" value="1" <?php checked( $this->_options['enable_post_new'], 1 ); ?> />
		<label for="tab_override_enable_post_new"><?php $is_version_3_plus ? esc_html_e( 'New Post / New Page', 'tab-override' ) : esc_html_e( 'New Post', 'tab-override' ); ?></label>
		<br />
		<?php if ( ! $is_version_3_plus ) : ?>
		<input type="checkbox" id="tab_override_enable_page" name="tab_override[enable_page]" value="1" <?php checked( $this->_options['enable_page'], 1 ); ?> />
		<label for="tab_override_enable_page"><?php esc_html_e( 'Page', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_page_new" name="tab_override[enable_page_new]" value="1" <?php checked( $this->_options['enable_page_new'], 1 ); ?> />
		<label for="tab_override_enable_page_new"><?php esc_html_e( 'New Page', 'tab-override' ); ?></label>
		<br />
		<?php endif; ?>
		<input type="checkbox" id="tab_override_enable_edit_comments" name="tab_override[enable_edit_comments]" value="1" <?php checked( $this->_options['enable_edit_comments'], 1 ); ?> />
		<label for="tab_override_enable_edit_comments"><?php esc_html_e( 'Edit Comments', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_comment" name="tab_override[enable_comment]" value="1" <?php checked( $this->_options['enable_comment'], 1 ); ?> />
		<label for="tab_override_enable_comment"><?php esc_html_e( 'Comment', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_theme_editor" name="tab_override[enable_theme_editor]" value="1" <?php checked( $this->_options['enable_theme_editor'], 1 ); ?> />
		<label for="tab_override_enable_theme_editor"><?php esc_html_e( 'Theme Editor', 'tab-override' ); ?></label>
		<br />
		<input type="checkbox" id="tab_override_enable_plugin_editor" name="tab_override[enable_plugin_editor]" value="1" <?php checked( $this->_options['enable_plugin_editor'], 1 ); ?> />
		<label for="tab_override_enable_plugin_editor"><?php esc_html_e( 'Plugin Editor', 'tab-override' ); ?></label>
		<?php
	}
	
	/**
	 * Outputs the HTML for the reset field.
	 */
	public function reset_field() {
		?>
		<input type="checkbox" id="tab_override_reset_options" name="tab_override[reset_options]" value="1" <?php checked( $this->_options['reset_options'], 1 ); ?> />
		<label for="tab_override_reset_options"><?php esc_html_e( 'Reset all options to defaults', 'tab-override' ); ?></label>
		<?php
	}
	
	/**
	 * Sanitizes option values before they are saved in the database.
	 *
	 * @param  array $input the settings from the submitted settings form
	 * @return array        the sanitized settings
	 */
	public function sanitize_settings( $input ) {
		$clean = array();
		
		$input_tab_size = trim( $input['tab_size'] );
		
		// the tab size must be either an empty string or a number between 1 and 100 (inclusive)
		if ( '' !== $input_tab_size ) {
			$input_tab_size = absint( $input_tab_size );
			
			if ( ! $input_tab_size || 100 < $input_tab_size ) {
				$input_tab_size = ''; // the default tab size
				
				if ( function_exists( 'add_settings_error' ) ) {
					add_settings_error(
						'tab_override_tab_size',
						'tab_override_tab_size_error',
						esc_html__( 'The tab size must be a number between 1 and 100.', 'tab-override' ),
						'error'
					);
				}
			}
		}
		
		$clean['tab_size'] = $input_tab_size;
		
		// these options are checkboxes which should be 1 if checked and 0 if not checked
		$checkboxes = array(
			'auto_indent',
			'enable_index',
			'enable_post',
			'enable_post_new',
			'enable_page',
			'enable_page_new',
			'enable_edit_comments',
			'enable_comment',
			'enable_theme_editor',
			'enable_plugin_editor',
			'reset_options'
		);
		
		foreach ( $checkboxes as $checkbox ) {
			$clean[$checkbox] = ( isset( $input[$checkbox] ) && 1 == $input[$checkbox] ) ? 1 : 0;
		}
		
		return $clean;
	}
	
	/**
	 * Registers the plugin settings and adds sections and fields to the
	 * settings page.
	 */
	public function register_settings() {
		// store all settings in an array
		register_setting( 'tab_override', 'tab_override',
			array( $this, 'sanitize_settings' ) );
		
		// add the settings section and fields
		add_settings_section(
			'editor',
			esc_html__( 'HTML Editor', 'tab-override' ),
			array( $this, 'editor_section_description' ),
			'tab-override-settings'
		);
		
		add_settings_field(
			'tab_override_tab_size',
			'<label for="tab_override_tab_size">' . esc_html__( 'Tab Size (number 1-100)', 'tab-override' ) . '</label>',
			array( $this, 'tab_size_field' ),
			'tab-override-settings',
			'editor'
		);
		
		add_settings_field(
			'tab_override_auto_indent',
			esc_html__( 'Auto Indent', 'tab-override' ),
			array( $this, 'auto_indent_field' ),
			'tab-override-settings',
			'editor'
		);
		
		add_settings_section(
			'general',
			esc_html__( 'General', 'tab-override' ),
			array( $this, 'general_section_description' ),
			'tab-override-settings'
		);
		
		add_settings_field(
			'tab_override_enable',
			esc_html__( 'Enable Tab Override on', 'tab-override' ),
			array( $this, 'enable_fields' ),
			'tab-override-settings',
			'general'
		);
		
		add_settings_field(
			'tab_override_reset_options',
			esc_html__( 'When plugin is activated', 'tab-override' ),
			array( $this, 'reset_field' ),
			'tab-override-settings',
			'general'
		);
	}
	
	/**
	 * Creates the settings menu.
	 */
	public function create_menu() {
		add_options_page( esc_html__( 'Tab Override Settings', 'tab-override' ), 'Tab Override',
			'manage_options', 'tab-override-settings', array( $this, 'create_settings_page' ) );
		
		add_action( 'admin_init', array( $this, 'register_settings' ) );
	}
	
	/**
	 * Adds the scripts to pages that have Tab Override enabled.
	 *
	 * @param string $hook_suffix the name of the current file being displayed
	 */
	public function add_scripts( $hook_suffix ) {
		// these are the admin pages that include a textarea for editing content
		$admin_edit_pages = array(
			'index.php', // Dashboard
			'post.php',
			'post-new.php',
			'page.php', // WP < 3.0
			'page-new.php', // WP < 3.0
			'edit-comments.php',
			'comment.php',
			'theme-editor.php',
			'plugin-editor.php'
		);
		
		if ( in_array( $hook_suffix, $admin_edit_pages ) ) {
			$settings = get_option( 'tab_override' );
			
			// check that this page is enabled in the options menu
			if ( $settings['enable_' . preg_replace( array( '/-/', '/\.php/' ), array( '_', '' ), $hook_suffix )] ) {
				
				// set the page type string
				switch ( $hook_suffix ) {
					case 'index.php':
						$page_type = 'dashboard';
						break;
					case 'post.php':
					case 'post-new.php':
						$page_type = 'post';
						break;
					case 'theme-editor.php':
					case 'plugin-editor.php':
						$page_type = 'editor';
						break;
					default:
						$page_type = 'general';
				}
				
				// the parameters used by the setup script
				// note that the JavaScript files use camelCase
				// the wp_localize_script function will convert any non-string values into strings
				// ex. true = "1", false = "", 4 = "4", '' = "", 'general' = "general"
				$params = array(
					'tabSize' => $settings['tab_size'],
					'autoIndent' => $settings['auto_indent'],
					'pageType' => $page_type
				);
				
				// include the Tab Override jQuery plugin in the footer
				wp_enqueue_script( 'tab-override', plugins_url( 'js/jquery.taboverride-1.1.min.js', $this->_plugin_file ),
					array( 'jquery' ), '1.1', true );
				
				// include the setup script in the footer
				wp_enqueue_script( 'tab-override-setup', plugins_url( 'js/tab-override-setup-1.0-min.js', $this->_plugin_file ),
					array( 'jquery', 'tab-override' ), '1.0', true );
				
				// add the setup parameters before the setup script
				wp_localize_script( 'tab-override-setup', 'tabOverrideParams', $params );
				
				// allow additional actions after scripts are enqueued
				do_action( 'tab_override_add_scripts' );
			}
		}
	}
}