// Tab Override Setup Script v1.0
// Part of the Tab Override WordPress plugin

/*jslint
    white: true, onevar: true, browser: true, undef: true, nomen: true,
    regexp: true, plusplus: true, bitwise: true, newcap: true, maxerr: 50,
    indent: 4
*/

/*global window, jQuery, tabOverrideParams, quickPressLoad: true, fullscreen: true */

(function ($) {
    var params = tabOverrideParams,
        quickPressLoadOrig,
        fullscreenOnOrig;
    
    // adds Tab Override to the quickPressLoad() function
    function quickPressLoadTabOverride() {
        quickPressLoadOrig();
        $('#content').tabOverride();
    }
    
    // adds Tab Override to the fullscreen.on() method
    function fullscreenOnTabOverride() {
        fullscreenOnOrig();
        $('#wp_mce_fullscreen').tabOverride();
    }
    
    // settings
    $.fn.tabOverride.setTabSize(parseInt(params.tabSize, 10));
    
    if (params.autoIndent) {
        $.fn.tabOverride.autoIndent = true;
    }
    
    // set Tab Override on the appropriate element(s) for the page type
    switch (params.pageType) {
    case 'dashboard':
        /*
        The QuickPress panel on the admin dashboard is now loaded via Ajax in WP 3.0.
        Setting Tab Override on window load will work only when the QuickPress panel
        is loaded by default. The global quickPressLoad function is executed when the
        panel is loaded. This is modified to include a call to Tab Override to ensure
        it is always set on the textarea regardless of when it is loaded.
        */
        $(window).bind('load.wpTabOverridePlugin', function () {
            quickPressLoadOrig = quickPressLoad;
            quickPressLoad = quickPressLoadTabOverride;
            $('#content, #replycontent').tabOverride();
        });
        break;
    case 'post':
        /*
        WordPress 3.2 introduced fullscreen mode for the HTML editor on the
        post and page edit pages. After the window is loaded, check for the
        global fullscreen object. If present, hook into the on() function and
        activate Tab Override for the $wp_mce_fullscreen textarea.
        */
        $('#content, #replycontent').tabOverride();
        $(window).bind('load.wpTabOverridePlugin', function () {
            if (window.fullscreen && fullscreen.on) {
                fullscreenOnOrig = fullscreen.on;
                fullscreen.on = fullscreenOnTabOverride;
            }
        });
        break;
    case 'editor':
        /*
        WordPress 3.0 added basic tabbing support to the theme and plugin editor
        pages. This interferes with Tab Override, so the corresponding event
        handlers must be removed before setting Tab Override on the textarea.
        This is done on window load since the event handlers are set up on documentready.
        WordPress 3.2 added namespacing for the events, so check for that first.
        */
        $(window).bind('load.wpTabOverridePlugin', function () {
            var newcontent = $('#newcontent'),
                eventListeners,
                eventTypes,
                numTypes,
                i,
                currType;
            
            // removes an event listener if it contains a specific piece of code
            function removeListeners(key, value) {
                // Older versions of jQuery stored the handler function as the value while
                // newer versions store it in value.handler. Both are checked to maintain
                // compatibility with older versions of WordPress. This code can be changed
                // once support for older versions is dropped:
                // var func = (typeof value.handler === 'function') ? value.handler : null;
                var func = (typeof value === 'function') ? value : ((typeof value.handler === 'function') ? value.handler : null);
                
                // the event listeners added by WP have lines referencing this.lastKey
                if (func && /this\.lastKey/.test(func.toString())) {
                    newcontent.unbind(currType, func);
                }
            }
            
            // remove all events with the wpevent_InsertTab namespace (WP 3.2+)
            newcontent.unbind('.wpevent_InsertTab');
            
            // check for any remaining event listeners
            // remove them if they are one of the specified types
            // this code can be removed when support is dropped for WP < 3.2
            eventListeners = newcontent.data('events');
            
            if (eventListeners) {
                eventTypes = ['keydown', 'blur'];
                numTypes = eventTypes.length;
                
                for (i = 0; i < numTypes; i += 1) {
                    currType = eventTypes[i];
                    if (eventListeners[currType]) {
                        $.each(eventListeners[currType], removeListeners);
                    }
                }
            }
            
            newcontent.tabOverride();
        });
        break;
    default:
        $('#content, #replycontent').tabOverride();
    }
}(jQuery));